from ndn.experiments.experiment import Experiment
from ndn.nlsr import Nlsr

import time
import sys

class ReadvertiseExperiment(Experiment):

  def __init__(self, args):
    Experiment.__init__(self, args)

  def run(self):
    endhost = next(host for host in self.net.hosts if host.name == "endhost")

    print("Starting pingserver on endhost.")
    # Start ping to trigger an AutoPrefixPropagation to the connected router.
    endhost.cmd("ndnpingserver /ndn/endhost/pingserver &")
    endhost.cmd("mkdir ping-data")

    # Give the gears some time to turn
    print("Allowing time for the prefix to propagate through NLSR.")
    time.sleep(10)

    router2 = next(host for host in self.net.hosts if host.name=="router2")

    wasPropagated = True
    notPropagatedHosts = []
    for host in self.net.hosts:
      if host.name != "endhost":
        fibContents = host.cmd("nfdc fib list | grep \"/ndn/endhost\"")
        wasPropagated = fibContents != ""
        if fibContents == "":
          notPropagatedHosts.push(host.name)

    if wasPropagated:
      print("/ndn/endhost prefix was successfully propagated.")
    else:
      print("/ndn/endhost prefix has not been successfully propagated.")
      print("The hosts that did not have the prefix are:")
      for host in notPropagatedHosts:
        print(host)

  def setup(self):
    for host in self.net.hosts:
      host.nfd.setStrategy("/ndn/", self.strategy)

    endhost = next(host for host in self.net.hosts if host.name=="endhost")
    # Kill host on the end host

    print("Killing NLSR on endhost...")
    NLSRpid = endhost.cmd("ps aux | tr -s \" \" | grep nlsr | grep endhost | cut -d' ' -f 2");
    endhost.cmd("kill " + NLSRpid)

    # Generate key for this host and export it to file
    print("Generating and exporting key on endhost.")
    endhost.cmd("ndnsec key-gen -i /ndn/" + endhost.name + " 2>&1")
    # We must use expect because ndnsec-export is interactive
    # Cert file ends up in endhost.ndnkey
    endhost.cmd("expect /tmp/export-cert.exp 2>&1")
    endhost.cmd("cp endhost.ndnkey /tmp/router1/ 2>&1")

    router1 = next(host for host in self.net.hosts if host.name=="router1")

    # Run the expect script to import the cert
    print("Importing endhost key on router1.")
    router1.cmd("perl /tmp/neighbor-remover.pl ~/nlsr.conf > ~/nlsr.conf.new")
    router1.cmd("mv ~/nlsr.conf ~/nlsr.conf.bak")
    router1.cmd("mv ~/nlsr.conf.new ~/nlsr.conf")

    print("Modifying router1's NFD config to enable AutoPrefixPropagation.")
    router1.cmd("perl /tmp/localhop-enabler.pl ~/router1.conf > ~/router1.conf.new")
    router1.cmd("mv ~/router1.conf ~/router1.conf.bak")
    router1.cmd("mv ~/router1.conf.new ~/router1.conf")

    # Remove the /tmp-identity key on router1
    router1TmpIdentityStr = router1.cmd("ndnsec list | grep tmp-identity")
    if router1TmpIdentityStr != "":
      router1StrToks = router1TmpIdentityStr.split()
      # print("temp identity on router1: " + router1TmpIdentityStr)
      router1.cmd("ndnsec delete " + router1StrToks[-1])

    router1.cmd("ndnsec set-default /ndn/router1-site/%C1.Router/cs/router1")

    # Remove the /tmp-identity key on endhost
    endhostTmpIdentityStr = endhost.cmd("ndnsec list | grep tmp-identity")
    if endhostTmpIdentityStr != "":
      endhostStrToks = endhostTmpIdentityStr.split()
      # print("temp identity on endhost: " + endhostTmpIdentityStr)
      endhost.cmd("ndnsec delete " + endhostStrToks[-1])

    endhost.cmd("ndnsec set-default /ndn/endhost")

    router1.cmd("expect /tmp/import-cert.exp 2>&1")
    print("Stopping NFD on router1 and endhost...")
    router1.nfd.stop()
    endhost.nfd.stop()
    time.sleep(5)

    print("Starting NFD on router1 and endhost...")
    router1.nfd.start()
    endhost.nfd.start()
    time.sleep(5)
    router1.nlsr.start()
    time.sleep(5)

    # for host in self.net.hosts:
    #   keys = host.cmd("ndnsec list")
    #   print("Keys on " + host.name + ":\n" + keys)

    # Add the link from endhost to router1
    print("Adding links from endhost to router1 for AutoPrefixPropagation.")
    links = endhost.connectionsTo(router1)

    endhost.cmd("nfdc register / udp4://" + endhost.IP(links[0][1]))
    endhost.cmd("nfdc register /localhop/nfd udp4://" + endhost.IP(links[0][1]) + " 2>&1")

    # Wait for convergence time period
    print "Waiting " + str(self.convergenceTime) + " seconds for convergence..."
    time.sleep(self.convergenceTime)
    print "...done"

    # To check whether all the nodes of NLSR have converged
    didNlsrConverge = True

    # Checking for convergence

    for host in self.net.hosts:
      # print host
        statusRouter = host.cmd("nfdc fib list | grep site/%C1.Router/cs/")
        statusPrefix = host.cmd("nfdc fib list | grep ndn | grep site | grep -v Router")
        # print(statusRouter)
        # print(statusPrefix)
        didNodeConverge = True
        if host.name == "endhost":
          continue
        # We know that "endhost" is at the front, so we cut from there to not check it.
        for node in self.nodes.split(",")[1:]:
            if ( ("/ndn/" + node + "-site/%C1.Router/cs/" + node) not in statusRouter or
                  str(host) != node and ("/ndn/" + node + "-site/" + node) not in statusPrefix ):
                didNodeConverge = False
                didNlsrConverge = False

        host.cmd("echo " + str(didNodeConverge) + " > convergence-result &")

    if didNlsrConverge:
        print("NLSR has successfully converged.")
    else:
        print("NLSR has not converged. Exiting...")
        self.net.stop()
        sys.exit(1)

Experiment.register("readvertise", ReadvertiseExperiment)
